@@ -67,9 +67,6 @@ public:
 
 	void setSelectedDifference( const Diff2::Difference* diff, bool scroll );
 
-	const QString& spaces() const { return m_spaces; };
-	void setSpaces( int spaces );
-
 public slots:
 	void slotSetSelection( const Diff2::DiffModel* model, const Diff2::Difference* diff );
 	void slotSetSelection( const Diff2::Difference* diff );
@@ -103,7 +100,6 @@ private:
 	int                               m_maxMainWidth;
 	const Diff2::DiffModel*           m_selectedModel;
 	const Diff2::Difference*          m_selectedDifference;
-	QString                           m_spaces;
 };
 
 class KompareListViewFrame : public Q3Frame
@@ -213,6 +209,9 @@ public:
 
 	KompareListViewDiffItem* diffItemParent() const;
 
+private:
+	void expandTabs(QString& text, int tabstop, int startPos = 0) const;
+
 private:
 	Diff2::DifferenceString* m_text;
 };
