@@ -1,11 +1,10 @@
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2007                                               *
+ *   copyright (C) 2002-2009                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
@@ -17,9 +16,9 @@
 #include <cmath>
 
 // qt includes
-#include <qcanvas.h>
-#include <qdatastream.h>
-#include <qdom.h>
+#include <QtXml/QDomDocument>
+#include <q3canvas.h>
+#include <QtGui/QPainter>
 
 // kde includes
 #include <kdebug.h>
@@ -32,56 +31,75 @@
 #include "umldoc.h"
 #include "uml.h"
 
-LinePath::Circle::Circle(QCanvas * canvas, int radius /* = 0 */)
-        : QCanvasEllipse(radius * 2, radius * 2, canvas) {
+LinePath::Circle::Circle(Q3Canvas * canvas, int radius /* = 0 */)
+        : Q3CanvasEllipse(radius * 2, radius * 2, canvas)
+{
 }
 
-void LinePath::Circle::setX(int x) {
-    QCanvasItem::setX( (double) x );
+void LinePath::Circle::setX(int x)
+{
+    Q3CanvasItem::setX( (double) x );
 }
 
-void LinePath::Circle::setY(int y) {
-    QCanvasItem::setY( (double) y );
+void LinePath::Circle::setY(int y)
+{
+    Q3CanvasItem::setY( (double) y );
 }
 
-void LinePath::Circle::setRadius(int radius) {
-    QCanvasEllipse::setSize(radius * 2, radius * 2);
+void LinePath::Circle::setRadius(int radius)
+{
+    Q3CanvasEllipse::setSize(radius * 2, radius * 2);
 }
 
-int LinePath::Circle::getRadius() const {
-    return (QCanvasEllipse::height() / 2);
+int LinePath::Circle::getRadius() const
+{
+    return (Q3CanvasEllipse::height() / 2);
 }
 
-void LinePath::Circle::drawShape(QPainter& p) {
+void LinePath::Circle::drawShape(QPainter& p)
+{
     int diameter = height();
     int radius = diameter / 2;
     p.drawEllipse( (int)x() - radius, (int)y() - radius, diameter, diameter);
 }
 
-LinePath::LinePath() {
-    m_RectList.setAutoDelete( true );
-    m_LineList.setAutoDelete( true );
-    m_HeadList.setAutoDelete( true );
-    m_ParallelList.setAutoDelete( true );
+/**
+ * Constructor.
+ */
+LinePath::LinePath()
+{
     m_bSelected = false;
     m_pClearPoly = 0;
     m_pCircle = 0;
+    m_pSubsetSymbol = 0;
     m_PointArray.resize( 4 );
     m_ParallelLines.resize( 4 );
     m_pAssociation = 0;
     m_bHeadCreated = false;
+    m_bSubsetSymbolCreated = false;
     m_bParallelLineCreated = false;
     m_DockRegion = TopBottom;
 }
 
-LinePath::~LinePath() {}
+/**
+ * Destructor.
+ */
+LinePath::~LinePath()
+{
+}
 
-void LinePath::setAssociation(AssociationWidget * association ) {
+/**
+ * This will setup the class ready to display the line correctly.
+ * This MUST be called before you can use this class.
+ */
+void LinePath::setAssociation(AssociationWidget * association)
+{
     if( !association )
         return;
     cleanup();
     m_pAssociation = association;
     createHeadLines();
+    createSubsetSymbol();
     if( getAssocType() == Uml::at_Coll_Message )
         setupParallelLine();
     UMLView * view =  (UMLView *)m_pAssociation -> parent();
@@ -89,26 +107,38 @@ void LinePath::setAssociation(AssociationWidget * association ) {
     connect( view, SIGNAL( sigLineWidthChanged( Uml::IDType ) ), this, SLOT( slotLineWidthChanged( Uml::IDType ) ) );
 }
 
-QPoint LinePath::getPoint( int pointIndex ) {
+/**
+ * Returns the point at the point index.
+ */
+QPoint LinePath::getPoint( int pointIndex ) const 
+{
     int count = m_LineList.count();
     if( count == 0 || pointIndex > count  || pointIndex < 0)
         return QPoint( -1, -1 );
 
     if( pointIndex == count ) {
-        QCanvasLine * line = m_LineList.last();
+        Q3CanvasLine * line = m_LineList.last();
         return line -> endPoint();
     }
-    QCanvasLine * line = m_LineList.at( pointIndex );
+    Q3CanvasLine * line = m_LineList.at( pointIndex );
     return line -> startPoint();
 }
 
-bool LinePath::setPoint( int pointIndex, const QPoint &point ) {
+/**
+ * Sets the position of an already set point.
+ */
+bool LinePath::setPoint( int pointIndex, const QPoint &point )
+{
     int count = m_LineList.count();
     if( count == 0 || pointIndex > count  || pointIndex < 0)
         return false;
+    if (point.x() == 0 && point.y() == 0) {
+        uError() << "ignoring request for (0,0)";
+        return false;
+    }
 
     if( pointIndex == count) {
-        QCanvasLine * line = m_LineList.last();
+        Q3CanvasLine * line = m_LineList.last();
         QPoint p = line -> startPoint();
         line -> setPoints( p.x(), p.y(), point.x(), point.y() );
         moveSelected( pointIndex );
@@ -116,14 +146,14 @@ bool LinePath::setPoint( int pointIndex, const QPoint &point ) {
         return true;
     }
     if( pointIndex == 0 ) {
-        QCanvasLine * line = m_LineList.first();
+        Q3CanvasLine * line = m_LineList.first();
         QPoint p = line -> endPoint();
         line -> setPoints( point.x(), point.y(), p.x(), p.y() );
         moveSelected( pointIndex );
         update();
         return true;
     }
-    QCanvasLine * line = m_LineList.at( pointIndex  );
+    Q3CanvasLine * line = m_LineList.at( pointIndex  );
     QPoint p = line -> endPoint();
     line -> setPoints( point.x(), point.y(), p.x(), p.y() );
     line = m_LineList.at( pointIndex - 1 );
@@ -134,12 +164,18 @@ bool LinePath::setPoint( int pointIndex, const QPoint &point ) {
     return true;
 }
 
-bool LinePath::isPoint( int pointIndex, const QPoint &point, unsigned short delta) {
+/**
+ * Checks, if we are at an end of the segment or somewhere in the middle.
+ * We use the delta, because with the mouse it is hard to find the
+ * exactly point.
+ */
+bool LinePath::isPoint( int pointIndex, const QPoint &point, unsigned short delta)
+{
     int count = m_LineList.count();
     if ( pointIndex >= count )
         return false;
 
-    QCanvasLine * line = m_LineList.at( pointIndex );
+    Q3CanvasLine * line = m_LineList.at( pointIndex );
 
     /* check if the given point is the start or end point of the line */
     if ( (
@@ -157,18 +193,22 @@ bool LinePath::isPoint( int pointIndex, const QPoint &point, unsigned short delt
     return false;
 }
 
-bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
+/**
+ * Inserts a point at the given index.
+ */
+bool LinePath::insertPoint( int pointIndex, const QPoint &point )
+{
     int count = m_LineList.count();
     if( count == 0 )
         return false;
     const bool bLoading = UMLApp::app()->getDocument()->loading();
 
     if( count == 1 || pointIndex == 1) {
-        QCanvasLine * first = m_LineList.first();
+        Q3CanvasLine * first = m_LineList.first();
         QPoint sp = first -> startPoint();
         QPoint ep = first -> endPoint();
         first -> setPoints( sp.x(), sp.y(), point.x(), point.y() );
-        QCanvasLine * line = new QCanvasLine( getCanvas() );
+        Q3CanvasLine * line = new Q3CanvasLine( getScene() );
         line -> setZ( -2 );
         line -> setPoints( point.x(), point.y(), ep.x(), ep.y() );
         line -> setPen( getPen() );
@@ -179,11 +219,11 @@ bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
         return true;
     }
     if( count + 1 == pointIndex ) {
-        QCanvasLine * before = m_LineList.last();
+        Q3CanvasLine * before = m_LineList.last();
         QPoint sp = before -> startPoint();
         QPoint ep = before -> endPoint();
         before -> setPoints( sp.x(), sp.y(), point.x(), point.y() );
-        QCanvasLine * line = new QCanvasLine( getCanvas() );
+        Q3CanvasLine * line = new Q3CanvasLine( getScene() );
         line -> setPoints( point.x(), point.y(), ep.x(), ep.y() );
         line -> setZ( -2 );
         line -> setPen( getPen() );
@@ -193,11 +233,11 @@ bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
             setupSelected();
         return true;
     }
-    QCanvasLine * before = m_LineList.at( pointIndex - 1 );
+    Q3CanvasLine * before = m_LineList.at( pointIndex - 1 );
     QPoint sp = before -> startPoint();
     QPoint ep = before -> endPoint();
     before -> setPoints( sp.x(), sp.y(), point.x(), point.y() );
-    QCanvasLine * line = new QCanvasLine(getCanvas() );
+    Q3CanvasLine * line = new Q3CanvasLine(getScene() );
     line -> setPoints( point.x(), point.y(), ep.x(), ep.y() );
     line -> setZ( -2 );
     line -> setPen( getPen() );
@@ -208,6 +248,10 @@ bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
     return true;
 }
 
+/**
+ * Removes the point on the line given by the index, at the coordinates
+ * given by point with a fuzzy of delta.
+ */
 bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short delta )
 {
     /* get the number of line segments */
@@ -217,7 +261,7 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
 
     /* we don't know if the user clicked on the start- or endpoint of a
      * line segment */
-    QCanvasLine * current_line = m_LineList.at( pointIndex );
+    Q3CanvasLine * current_line = m_LineList.at( pointIndex );
     if (abs( current_line -> endPoint().x() - point.x() ) <= delta
             &&
             abs( current_line -> endPoint().y() - point.y() ) <= delta)
@@ -229,7 +273,7 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
 
         /* the next segment will get the starting point from the current one,
          * which is going to be removed */
-        QCanvasLine * next_line = m_LineList.at( pointIndex + 1 );
+        Q3CanvasLine * next_line = m_LineList.at( pointIndex + 1 );
         QPoint startPoint = current_line -> startPoint();
         QPoint endPoint = next_line -> endPoint();
         next_line -> setPoints(startPoint.x(), startPoint.y(),
@@ -247,7 +291,7 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
 
             /* the previous segment will get the end point from the current one,
              * which is going to be removed */
-            QCanvasLine * previous_line = m_LineList.at( pointIndex - 1 );
+            Q3CanvasLine * previous_line = m_LineList.at( pointIndex - 1 );
             QPoint startPoint = previous_line -> startPoint();
             QPoint endPoint = current_line -> endPoint();
             previous_line -> setPoints(startPoint.x(), startPoint.y(),
@@ -258,19 +302,21 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
             return false;
         }
 
-
     /* remove the segment from the list */
-    m_LineList.remove( pointIndex );
+    delete m_LineList.takeAt( pointIndex );
 
     return true;
 }
 
-bool LinePath::setStartEndPoints( const QPoint &start, const QPoint &end ) {
+/**
+ * Sets the start and end points.
+ */
+bool LinePath::setStartEndPoints( const QPoint &start, const QPoint &end )
+{
     int count = m_LineList.count();
-
     if( count == 0 ) {
-        QCanvasLine * line = new QCanvasLine(getCanvas() );
-        line -> setPoints( start.x(), start.y(), end.x(), end.y() );
+        Q3CanvasLine * line = new Q3CanvasLine(getScene() );
+        line -> setPoints( start.x(), start.y(),end.x(),end.y() );
         line -> setZ( -2 );
         line -> setPen( getPen() );
         line -> setVisible( true );
@@ -279,52 +325,80 @@ bool LinePath::setStartEndPoints( const QPoint &start, const QPoint &end ) {
     }
     bool status = setPoint( 0, start );
     if( status)
-        return setPoint( count , end );
+        return setPoint(count,end);
     return false;
 }
 
-int LinePath::count() {
+/**
+ * Returns the amount of POINTS on the line.
+ * Includes start and end points.
+ */
+int LinePath::count() const 
+{
     return m_LineList.count() + 1;
 }
 
-int LinePath::onLinePath( const QPoint &position ) {
-    QCanvasItemList list = getCanvas() -> collisions( position );
+/**
+ * Returns -1 if the given point is not on the line.
+ * else returns the line segment the point is on.
+ * Use the value to insert points at the point position.
+ */
+int LinePath::onLinePath( const QPoint &position )
+{
+    Q3CanvasItemList list = getScene()->collisions( position );
     int index = -1;
 
-    QCanvasItemList::iterator end(list.end());
-    for(QCanvasItemList::iterator item_it(list.begin()); item_it != end; ++item_it ) {
-        if( ( index = m_LineList.findRef( (QCanvasLine*)*item_it ) ) != -1 )
+    Q3CanvasItemList::iterator end(list.end());
+    for(Q3CanvasItemList::iterator item_it(list.begin()); item_it != end; ++item_it ) {
+        if( ( index = m_LineList.indexOf( (Q3CanvasLine*)*item_it ) ) != -1 )
             break;
     }//end for
     return index;
 }
 
-void LinePath::setSelected( bool select ) {
-    if(select)
+/**
+ * Sets the status of whether the line is selected or not.
+ */
+void LinePath::setSelected( bool select ) 
+{
+    if(select) {
         setupSelected();
-    else if( m_RectList.count() > 0 )
+    }
+    else if(!m_RectList.isEmpty()) {
+        qDeleteAll(m_RectList);
         m_RectList.clear();
+    }
 }
 
-void LinePath::setAssocType( Uml::Association_Type type ) {
-    LineListIt it( m_LineList );
-    QCanvasLine * line = 0;
-    while( ( line = it.current() ) ) {
-        line -> setPen( getPen() );
-        ++it;
-    }
-    if( m_pClearPoly ) {
-        delete m_pClearPoly;
-        m_pClearPoly = 0;
-    }
-    if( type == Uml::at_Coll_Message )
+/**
+ * Sets the Association type.
+ */
+void LinePath::setAssocType( Uml::Association_Type type )
+{
+    QList<Q3CanvasLine*>::Iterator it = m_LineList.begin();
+    QList<Q3CanvasLine*>::Iterator end = m_LineList.end();
+
+    for( ; it != end; ++it )
+        (*it) -> setPen( getPen() );
+
+    delete m_pClearPoly;
+    m_pClearPoly = 0;
+
+    if( type == Uml::at_Coll_Message ) {
         setupParallelLine();
-    else
+    }
+    else {
         createHeadLines();
+        createSubsetSymbol();
+    }
     update();
 }
 
-void LinePath::update() {
+/**
+ * Calls a group of methods to update the line. Used to save you calling multiple methods.
+ */
+void LinePath::update()
+{
     if (getAssocType() == Uml::at_Coll_Message) {
         if (m_bParallelLineCreated) {
             calculateParallelLine();
@@ -337,42 +411,60 @@ void LinePath::update() {
     } else {
         createHeadLines();
     }
+
+    if ( m_bSubsetSymbolCreated ) {
+        updateSubsetSymbol();
+    } else {
+        createSubsetSymbol();
+    }
 }
 
-void LinePath::slotLineColorChanged( Uml::IDType viewID ) {
-    if(m_pAssociation->getUMLView()->getID() != viewID) {
+/**
+ * Sets the line color used by the line.
+ *
+ * @param viewID The id of the object behind the widget.
+ */
+void LinePath::slotLineColorChanged( Uml::IDType viewID )
+{
+    if(m_pAssociation->umlScene()->getID() != viewID) {
         return;
     }
-    setLineColor( m_pAssociation->getUMLView()->getLineColor() );
+    setLineColor( m_pAssociation->umlScene()->getLineColor() );
 }
 
-
-void LinePath::setLineColor( const QColor &color ) {
-    QCanvasLine * line = 0;
+/**
+ * Sets the line color used by the line.
+ */
+void LinePath::setLineColor( const QColor &color )
+{
     uint linewidth = 0;
-    LineListIt it( m_LineList );
-    while( ( line = it.current() ) ) {
+    Q3CanvasLine * line = 0;
+
+    Q_FOREACH( line, m_LineList ) {
         linewidth = line->pen().width();
-        line -> setPen( QPen( color, linewidth ) );
-        ++it;
-    }
-    LineListIt hit( m_HeadList );
-    while( ( line = hit.current() ) ) {
-        linewidth = line->pen().width();
-        line -> setPen( QPen( color, linewidth ) );
-        ++hit;
-    }
-    LineListIt pit( m_ParallelList );
-    while( ( line = pit.current() ) ) {
-        linewidth = line->pen().width();
-        line -> setPen( QPen( color, linewidth ) );
-        ++pit;
+        line->setPen( QPen( color, linewidth ) );
     }
 
-    if( getAssocType() == Uml::at_Aggregation )
-        if (m_pClearPoly) m_pClearPoly -> setBrush( QBrush( Qt::white ) );
-        else if( getAssocType() == Uml::at_Composition )
-            if (m_pClearPoly) m_pClearPoly -> setBrush( QBrush( color ) );
+    Q_FOREACH( line, m_HeadList ) {
+        linewidth = line->pen().width();
+        line->setPen( QPen( color, linewidth ) );
+    }
+
+    Q_FOREACH( line, m_ParallelList ) {
+        linewidth = line->pen().width();
+        line->setPen( QPen( color, linewidth ) );
+    }
+
+    if (getAssocType() == Uml::at_Aggregation) {
+        if (m_pClearPoly) {
+            m_pClearPoly->setBrush( QBrush( Qt::white ) );
+        }
+        else if( getAssocType() == Uml::at_Composition ) {
+            if (m_pClearPoly) {
+                m_pClearPoly->setBrush( QBrush( color ) );
+            }
+        }
+    }
 
     if( m_pCircle ) {
         linewidth = m_pCircle->pen().width();
@@ -380,33 +472,41 @@ void LinePath::setLineColor( const QColor &color ) {
     }
 }
 
-void LinePath::slotLineWidthChanged( Uml::IDType viewID ) {
-    if(m_pAssociation->getUMLView()->getID() != viewID) {
+/**
+ * Sets the line width used by the line.
+ *
+ * @param viewID The id of the object behind the widget.
+ */
+void LinePath::slotLineWidthChanged( Uml::IDType viewID )
+{
+    if(m_pAssociation->umlScene()->getID() != viewID) {
         return;
     }
-    setLineWidth( m_pAssociation->getUMLView()->getLineWidth() );
+    setLineWidth( m_pAssociation->umlScene()->getLineWidth() );
 }
 
-void LinePath::setLineWidth( uint width ) {
-    QCanvasLine * line = 0;
+/**
+ * Sets the line width used by the line.
+ * @param width   the new width of the line
+ */
+void LinePath::setLineWidth( uint width )
+{
     QColor linecolor;
-    LineListIt it( m_LineList );
-    while( ( line = it.current() ) ) {
+    Q3CanvasLine * line = 0;
+
+    Q_FOREACH( line, m_LineList ) {
         linecolor = line->pen().color();
-        line -> setPen( QPen( linecolor, width ) );
-        ++it;
+        line->setPen( QPen( linecolor, width ) );
     }
-    LineListIt hit( m_HeadList );
-    while( ( line = hit.current() ) ) {
+
+    Q_FOREACH( line, m_HeadList ) {
         linecolor = line->pen().color();
-        line -> setPen( QPen( linecolor, width ) );
-        ++hit;
+        line->setPen( QPen( linecolor, width ) );
     }
-    LineListIt pit( m_ParallelList );
-    while( ( line = pit.current() ) ) {
+
+    Q_FOREACH( line, m_ParallelList ) {
         linecolor = line->pen().color();
-        line -> setPen( QPen( linecolor, width ) );
-        ++pit;
+        line->setPen( QPen( linecolor, width ) );
     }
 
     if( m_pCircle ) {
@@ -415,16 +515,21 @@ void LinePath::setLineWidth( uint width ) {
     }
 }
 
-void LinePath::moveSelected( int pointIndex ) {
+/**
+ * Moves the selected canvas widgets.
+ */
+void LinePath::moveSelected( int pointIndex )
+{
     int lineCount = m_LineList.count();
     if( !m_bSelected ) {
+        qDeleteAll( m_RectList.begin(), m_RectList.end() );
         m_RectList.clear();
         return;
     }
     if( (int)m_RectList.count() + 1 != lineCount )
         setupSelected();
-    QCanvasRectangle * rect = 0;
-    QCanvasLine * line = 0;
+    Q3CanvasRectangle * rect = 0;
+    Q3CanvasLine * line = 0;
     if( pointIndex == lineCount || lineCount == 1) {
         line = m_LineList.last();
         QPoint p = line -> endPoint();
@@ -442,32 +547,44 @@ void LinePath::moveSelected( int pointIndex ) {
     rect -> setZ( 4 );
 }
 
-void LinePath::setupSelected() {
+/**
+ * Sets up the selected canvases needed.
+ */
+void LinePath::setupSelected()
+{
+    qDeleteAll( m_RectList.begin(), m_RectList.end() );
     m_RectList.clear();
-    QCanvasLine * line = 0;
-    LineListIt it( m_LineList );
-    while( ( line = it.current() ) ) {
+    Q3CanvasLine * line = 0;
+
+    Q_FOREACH( line, m_LineList ) {
         QPoint sp = line -> startPoint();
-        QCanvasRectangle *rect = Widget_Utils::decoratePoint(sp);
+        Q3CanvasRectangle *rect = Widget_Utils::decoratePoint(sp);
         m_RectList.append( rect );
-        ++it;
     }
     //special case for last point
     line = m_LineList.last();
     QPoint p = line -> endPoint();
-    QCanvasRectangle *rect = Widget_Utils::decoratePoint(p);
+    Q3CanvasRectangle *rect = Widget_Utils::decoratePoint(p);
     m_RectList.append( rect );
     update();
 }
 
-QPen LinePath::getPen() {
+/**
+ * Returns the type of pen to use depending on the type of Association.
+ */
+QPen LinePath::getPen()
+{
     Uml::Association_Type type = getAssocType();
     if( type == Uml::at_Dependency || type == Uml::at_Realization || type == Uml::at_Anchor )
         return QPen( getLineColor(), getLineWidth(), Qt::DashLine );
     return QPen( getLineColor(), getLineWidth() );
 }
 
-void LinePath::calculateHead() {
+/**
+ * Calculates the head points.
+ */
+void LinePath::calculateHead()
+{
     uint size = m_LineList.count();
     QPoint farPoint;
     int halfLength = 10;
@@ -549,17 +666,23 @@ void LinePath::calculateHead() {
 
 }
 
-void LinePath::updateHead() {
+/**
+ * Updates the head lines. Call after calculating the new points.
+ */
+void LinePath::updateHead()
+{
     int count = m_HeadList.count();
-    QCanvasLine * line = 0;
+    Q3CanvasLine * line = 0;
 
     switch( getAssocType() ) {
     case Uml::at_State:
     case Uml::at_Activity:
+    case Uml::at_Exception:
     case Uml::at_UniAssociation:
     case Uml::at_Dependency:
         if( count < 2)
             return;
+
         line = m_HeadList.at( 0 );
         line -> setPoints( m_EgdePoint.x(), m_EgdePoint.y(), m_ArrowPointA.x(), m_ArrowPointA.y() );
 
@@ -634,10 +757,17 @@ void LinePath::updateHead() {
     }
 }
 
-void LinePath::growList(LineList &list, int by) {
+/**
+ * Create a number of new lines and append them to the given list.
+ *
+ * @param list  The list into which to append lines.
+ * @param by    The number of lines to insert into the given list.
+ */
+void LinePath::growList(LineList &list, int by)
+{
     QPen pen( getLineColor(), getLineWidth() );
     for (int i = 0; i < by; i++) {
-        QCanvasLine * line = new QCanvasLine( getCanvas() );
+        Q3CanvasLine * line = new Q3CanvasLine( getScene() );
         line -> setZ( 0 );
         line -> setPen( pen );
         line -> setVisible( true );
@@ -645,11 +775,17 @@ void LinePath::growList(LineList &list, int by) {
     }
 }
 
-void LinePath::createHeadLines() {
+/**
+ * Creates the head lines to display the head.
+ */
+void LinePath::createHeadLines()
+{
+    qDeleteAll( m_HeadList.begin(), m_HeadList.end() );
     m_HeadList.clear();
-    QCanvas * canvas = getCanvas();
+    Q3Canvas * canvas = getScene();
     switch( getAssocType() ) {
     case Uml::at_Activity:
+    case Uml::at_Exception:
     case Uml::at_State:
     case Uml::at_Dependency:
     case Uml::at_UniAssociation:
@@ -660,7 +796,7 @@ void LinePath::createHeadLines() {
     case Uml::at_Generalization:
     case Uml::at_Realization:
         growList(m_HeadList, 3);
-        m_pClearPoly = new QCanvasPolygon( canvas );
+        m_pClearPoly = new Q3CanvasPolygon( canvas );
         m_pClearPoly -> setVisible( true );
         m_pClearPoly -> setBrush( QBrush( Qt::white ) );
         m_pClearPoly -> setZ( -1 );
@@ -669,10 +805,10 @@ void LinePath::createHeadLines() {
     case Uml::at_Composition:
     case Uml::at_Aggregation:
         growList(m_HeadList, 4);
-        m_pClearPoly = new QCanvasPolygon( canvas );
+        m_pClearPoly = new Q3CanvasPolygon( canvas );
         m_pClearPoly -> setVisible( true );
         if( getAssocType() == Uml::at_Aggregation )
-            m_pClearPoly -> setBrush( QBrush( Qt::white ) );
+            m_pClearPoly->setBrush( QBrush( Qt::white ) );
         else
             m_pClearPoly -> setBrush( QBrush( getLineColor() ) );
         m_pClearPoly -> setZ( -1 );
@@ -692,7 +828,11 @@ void LinePath::createHeadLines() {
     m_bHeadCreated = true;
 }
 
-void LinePath::calculateParallelLine() {
+/**
+ * Calculates the position of the parallel line.
+ */
+void LinePath::calculateParallelLine()
+{
     int midCount = count() / 2;
     double ATAN = atan(1.0);
     int lineDist = 10;
@@ -740,16 +880,26 @@ void LinePath::calculateParallelLine() {
     m_ParallelLines[ 3 ] = QPoint( pointT.x() - (int)cosx, pointT.y() - (int)siny );
 }
 
-void LinePath::setupParallelLine() {
+/**
+ * Creates the line objects to display the parallel line.
+ */
+void LinePath::setupParallelLine()
+{
+    qDeleteAll( m_ParallelList.begin(), m_ParallelList.end() );
     m_ParallelList.clear();
     growList(m_ParallelList, 3);
     m_bParallelLineCreated = true;
 }
 
-void LinePath::updateParallelLine() {
+/**
+ * Updates the parallel line.
+ * Call after calculating the new position.
+ */
+void LinePath::updateParallelLine() 
+{
     if( !m_bParallelLineCreated )
         return;
-    QCanvasLine * line = 0;
+    Q3CanvasLine * line = 0;
     QPoint common = m_ParallelLines.at( 0 );
     QPoint p = m_ParallelLines.at( 1 );
     line = m_ParallelList.at( 0 );
@@ -764,84 +914,137 @@ void LinePath::updateParallelLine() {
     line -> setPoints( common.x(), common.y(), p.x(), p.y() );
 }
 
-bool LinePath::operator==( LinePath & rhs ) {
-    if( this -> m_LineList.count() != rhs.m_LineList.count() )
+/**
+ * Equal to (==) operator.
+ */
+bool LinePath::operator==( const LinePath & rhs ) 
+{
+    if( this->m_LineList.count() != rhs.m_LineList.count() )
         return false;
 
     //Check to see if all points at the same position
     for( int i = 0; i< rhs.count() ; i++ ) {
-        if( this -> getPoint( i ) != rhs.getPoint( i ) )
+        if( this->getPoint( i ) != rhs.getPoint( i ) )
             return false;
     }
     return true;
 }
 
-LinePath & LinePath::operator=( LinePath & rhs ) {
+/**
+ * Copy ( = ) operator.
+ */
+LinePath & LinePath::operator=( const LinePath & rhs )
+{
     if( this == &rhs )
         return *this;
     //clear out the old canvas objects
-    this -> m_LineList.clear();
-    this -> m_ParallelList.clear();
-    this -> m_RectList.clear();
-    this -> m_HeadList.clear();
+    this->cleanup();
+
     int count = rhs.m_LineList.count();
     //setup start end points
-    this -> setStartEndPoints( rhs.getPoint( 0 ), rhs.getPoint( count) );
+    this->setStartEndPoints( rhs.getPoint( 0 ), rhs.getPoint( count) );
     //now insert the rest
     for( int i = 1; i < count ; i++ ) {
-        this -> insertPoint( i, rhs.getPoint ( i ) );
+        this->insertPoint( i, rhs.getPoint ( i ) );
     }
-    this -> setAssocType( rhs.getAssocType() );
+    this->setAssocType( rhs.getAssocType() );
 
     return *this;
 }
 
-QCanvas * LinePath::getCanvas() {
+/**
+ * Returns the canvas being used.
+ * Will return zero if the Association hasn't been set.
+ *
+ * This class doesn't hold this information but is a wrapper
+ * method to stop calls to undefined variable like m_pAssociation.
+ */
+Q3Canvas * LinePath::getScene()
+{
     if( !m_pAssociation )
         return 0;
-    const UMLView * view =  m_pAssociation->getUMLView();
-    return view -> canvas();
+    const UMLView * view =  m_pAssociation->umlScene();
+    return view->canvas();
 }
 
-Uml::Association_Type LinePath::getAssocType() {
+/**
+ * Returns the Association type.
+ * Returns Uml::at_Association if association hasn't been set.
+ *
+ * This class doesn't hold this information but is a wrapper
+ * method to stop calls to undefined variable like m_pAssociation.
+ */
+Uml::Association_Type LinePath::getAssocType() const 
+{
     if( m_pAssociation )
-        return m_pAssociation -> getAssocType();
+        return m_pAssociation->getAssocType();
     return Uml::at_Association;
 }
 
-QColor LinePath::getLineColor() {
+/**
+ * Returns the Line Color to use.
+ * Returns black if association not set.
+ *
+ * This class doesn't hold this information but is a wrapper
+ * method to stop calls to undefined variable like m_pAssociation.
+ */
+QColor LinePath::getLineColor()
+{
     if( !m_pAssociation )
         return Qt::black;
-    return m_pAssociation -> getLineColor();
+    return m_pAssociation->lineColor();
 }
 
-uint LinePath::getLineWidth() {
+/**
+ * Returns the Line Width to use.
+ * Returns 0 if association not set.
+ *
+ * This class doesn't hold this information but is a wrapper
+ * method to stop calls to undefined variable like m_pAssociation.
+ */
+uint LinePath::getLineWidth()
+{
     if( !m_pAssociation )
         return 0;
-    int viewLineWidth = m_pAssociation -> getLineWidth();
+    int viewLineWidth = m_pAssociation->lineWidth();
     if ( viewLineWidth >= 0 && viewLineWidth <= 10 )
         return viewLineWidth;
     else {
-        kWarning() << "Ignore wrong LineWidth of " << viewLineWidth
-        << " in LinePath::getLineWidth" << endl;
+        uWarning() << "Ignore wrong LineWidth of " << viewLineWidth
+                   << " in LinePath::getLineWidth";
         return 0;
     }
 }
 
-void LinePath::cleanup() {
-    if (m_pAssociation)
+/**
+ * Removes and item created that are no longer needed.
+ */
+void LinePath::cleanup()
+{
+    if (m_pAssociation) {
+        qDeleteAll( m_LineList.begin(), m_LineList.end() );
         m_LineList.clear();
+    }
+
+    qDeleteAll( m_HeadList.begin(), m_HeadList.end() );
     m_HeadList.clear();
+
+    qDeleteAll( m_RectList.begin(), m_RectList.end() );
     m_RectList.clear();
+
+    qDeleteAll( m_ParallelList.begin(), m_ParallelList.end() );
     m_ParallelList.clear();
 
     if( m_pClearPoly )
         delete m_pClearPoly;
     if( m_pCircle )
         delete m_pCircle;
+    if( m_pSubsetSymbol )
+        delete m_pSubsetSymbol;
     m_pCircle = 0;
     m_pClearPoly = 0;
-    m_bHeadCreated = m_bParallelLineCreated = false;
+    m_pSubsetSymbol = 0;
+    m_bHeadCreated = m_bParallelLineCreated = m_bSubsetSymbolCreated = false;
     if( m_pAssociation ) {
         UMLView * view =  (UMLView *)m_pAssociation -> parent();
         if(view) {
@@ -852,25 +1055,33 @@ void LinePath::cleanup() {
     }
 }
 
-void LinePath::setDockRegion( Region region ) {
+/**
+ * Tell the line where the line docks.
+ */
+void LinePath::setDockRegion( Region region )
+{
     m_DockRegion = region;
 }
 
-bool LinePath::hasPoints () {
+bool LinePath::hasPoints () const
+{
     int count = m_LineList.count();
     if (count>1)
         return true;
     return false;
 }
-void LinePath::dumpPoints () {
+
+void LinePath::dumpPoints ()
+{
     int count = m_LineList.count();
     for( int i = 1; i < count; i++ ) {
         QPoint point = getPoint( i );
-        kDebug()<<" * point x:"<<point.x()<<" y:"<<point.y()<<endl;
+        uDebug()<<" * point x:"<<point.x()<<" y:"<<point.y();
     }
 }
 
-void LinePath::saveToXMI( QDomDocument & qDoc, QDomElement & qElement ) {
+void LinePath::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
+{
     int count = m_LineList.count();
     QPoint point = getPoint( 0 );
     QDomElement lineElement = qDoc.createElement( "linepath" );
@@ -893,7 +1104,8 @@ void LinePath::saveToXMI( QDomDocument & qDoc, QDomElement & qElement ) {
     qElement.appendChild( lineElement );
 }
 
-bool LinePath::loadFromXMI( QDomElement & qElement ) {
+bool LinePath::loadFromXMI( QDomElement & qElement )
+{
     QDomNode node = qElement.firstChild();
     QDomElement startElement = node.toElement();
     if( startElement.isNull() || startElement.tagName() != "startpoint" )
@@ -933,21 +1145,109 @@ bool LinePath::loadFromXMI( QDomElement & qElement ) {
     return true;
 }
 
-
-void LinePath::activate() {
+/**
+ * Activates the line list.
+ * This is needed because the m_pAssociation does not yet
+ * exist at the time of the LinePath::loadFromXMI call.
+ * However, this means that the points in the m_LineList
+ * do not have a parent when they are loaded.
+ * They need to be reparented by calling LinePath::activate()
+ * once the m_pAssociation exists.
+ */
+void LinePath::activate()
+{
     int count = m_LineList.count();
     if (count == 0)
         return;
-    QCanvas * canvas = getCanvas();
+    Q3Canvas * canvas = getScene();
     if (canvas == NULL)
         return;
     for (int i = 0; i < count ; i++) {
-        QCanvasLine *line = m_LineList.at(i);
+        Q3CanvasLine *line = m_LineList.at(i);
         line -> setCanvas( canvas );
         line -> setPen( getPen() );
     }
 }
 
+/**
+ * Creates the subset symbol.
+ */
+void LinePath::createSubsetSymbol()
+{
+    if ( m_LineList.count() < 1 ) {
+        return;
+    }
 
+    switch( getAssocType() ) {
+       case Uml::at_Child2Category:
+           m_pSubsetSymbol = new SubsetSymbol(getScene());
+           m_pSubsetSymbol->setPen( QPen( getLineColor(), getLineWidth() ) );
+           updateSubsetSymbol();
+           m_pSubsetSymbol->show();
+           break;
+       default:
+           break;
+    }
+    m_bSubsetSymbolCreated = true;
+}
+
+
+/**
+ * Updates the subset symbol.Call after calculating the new points.
+ */
+void LinePath::updateSubsetSymbol()
+{
+    if ( m_LineList.count() < 1 ) {
+        return;
+    }
+    Q3CanvasLine* firstLine = m_LineList.first();
+    QPoint startPoint = firstLine->startPoint();
+    QPoint endPoint = firstLine->endPoint();
+    QPoint centrePoint;
+    centrePoint.setX( ( startPoint.x() + endPoint.x() )/2 );
+    centrePoint.setY( ( startPoint.y() + endPoint.y() )/2 );
+
+    if ( m_pSubsetSymbol ) {
+
+        double xDiff = endPoint.x() - startPoint.x();
+        double yDiff = endPoint.y() - startPoint.y();
+
+        int inclination;
+        if ( xDiff == 0 ) {
+            if ( yDiff > 0 )
+                inclination = 90;
+            else // yDiff < 0
+                inclination = 270;
+        } else {
+            inclination = (int)(atan( yDiff/xDiff )*180/3.14159) ;
+            // convert to 360 degree scale
+            if (  xDiff < 0 ) {
+                inclination = 180 + inclination ;
+            } else if ( xDiff > 0 && yDiff < 0 ) {
+                inclination = 360 +  inclination;
+            }
+        }
+
+        m_pSubsetSymbol->setInclination(inclination);
+        m_pSubsetSymbol->setX(centrePoint.x());
+        m_pSubsetSymbol->setY(centrePoint.y());
+    }
+}
+
+LinePath::SubsetSymbol::SubsetSymbol(Q3Canvas* canvas)
+    : Q3CanvasEllipse(canvas) {
+    inclination = 0;
+}
+
+void LinePath::SubsetSymbol::drawShape(QPainter& p) {
+    p.translate(QPoint( ( int )x(), ( int )y() ) );
+    p.rotate( inclination );
+    int width = 30, height = 20;
+    int startAngle = 90, endAngle = 180;
+    p.drawArc( 0 ,-height/2, width, height, startAngle*16, endAngle*16 );
+    // revert back
+    p.rotate( -inclination );
+    p.translate( QPoint( ( int )-x(), ( int )-y() ) );
+}
 
 #include "linepath.moc"
