@@ -1,15 +1,11 @@
-/*
- *  copyright (C) 2004
- *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
- */
-
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
+ *   copyright (C) 2004-2008                                               *
+ *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef TOOLBARSTATEOTHER_H
@@ -17,30 +13,30 @@
 
 #include "toolbarstatepool.h"
 
-
-class QMouseEvent;
-class UMLView;
-class UMLWidget;
-
 /**
- * Almost every object is created by this state. Except assiociations and
- * messages. 
- */ 
+ * Other tool creates almost all the objects (except associations and messages).
+ * Objects are created when left button is released, no matter if it was
+ * released on an association, on a widget or on an empty space.
+ *
+ * Associations and widgets aren't taken into account, and are treated as empty
+ * spaces.
+ */
 class ToolBarStateOther : public ToolBarStatePool
 {
-
+    Q_OBJECT
 public:
+
     ToolBarStateOther(UMLView *umlView);
     virtual ~ToolBarStateOther();
 
-    virtual void mousePress(QMouseEvent* ome);
-    virtual void mouseRelease(QMouseEvent* ome);
-    virtual void mouseDoubleClick(QMouseEvent* ome);
-    virtual void mouseMove(QMouseEvent* ome);
-
 private:
 
-    Uml::Object_Type getObjectType(WorkToolBar::ToolBar_Buttons tbb);
+    virtual void setCurrentElement();
+
+    virtual void mouseReleaseEmpty();
+
+    Uml::Object_Type getObjectType();
+
     bool newWidget();
 
 };
